"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoBoundWitnessValidator = void 0;
class XyoBoundWitnessValidator {
    constructor(signatureValidators) {
        this.signatureValidators = signatureValidators;
    }
    validate(boundWitness, allowUnknownSignatures) {
        const signingData = boundWitness.getSigningData();
        const publicKeysParty = boundWitness.getPublicKeys();
        const signaturesParty = boundWitness.getSignatures();
        if (publicKeysParty.length !== signaturesParty.length) {
            return false;
        }
        for (let i = 0; i < publicKeysParty.length; i++) {
            const publicKeys = publicKeysParty[i];
            const signatures = signaturesParty[i];
            if (publicKeysParty.length !== signaturesParty.length) {
                return false;
            }
            for (let j = 0; j < publicKeys.length; j++) {
                const publicKey = publicKeys[j];
                const signature = signatures[j];
                const signatureValidator = this.signatureValidators.get(signature.getSchema().id);
                if (signatureValidator) {
                    if (!signatureValidator(publicKey.getAll().getContentsCopy(), signature.getAll().getContentsCopy(), signingData)) {
                        return false;
                    }
                }
                if (signatureValidator && !allowUnknownSignatures) {
                    return false;
                }
            }
        }
        return true;
    }
}
exports.XyoBoundWitnessValidator = XyoBoundWitnessValidator;
//# sourceMappingURL=xyo-bound-witness-validator.js.map