"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoBoundWitness = void 0;
const object_model_1 = require("../object-model");
const schema_1 = require("../schema");
class XyoBoundWitness extends object_model_1.XyoIterableStructure {
    static createMasterArrayWithSubArray(masterSchema, subSchema, masterItems, subItems) {
        const sub = object_model_1.XyoIterableStructure.newIterable(subSchema, subItems);
        const itemsInMaster = [sub];
        for (const materItem of masterItems) {
            itemsInMaster.push(materItem);
        }
        return object_model_1.XyoIterableStructure.newIterable(masterSchema, itemsInMaster);
    }
    getIsCompleted() {
        if (this.getId(schema_1.XyoObjectSchema.WITNESS.id).length !== 0) {
            return (this.getId(schema_1.XyoObjectSchema.WITNESS.id).length ===
                this.getId(schema_1.XyoObjectSchema.FETTER.id).length);
        }
        return false;
    }
    // [party index][key index]
    getPublicKeys() {
        const keysets = [];
        const fetters = this.getId(schema_1.XyoObjectSchema.FETTER.id);
        for (const fetter of fetters) {
            const keyset = [];
            const partyKeySets = fetter.getId(schema_1.XyoObjectSchema.KEY_SET.id);
            for (const partyKeySet of partyKeySets) {
                const keysetIt = partyKeySet.newIterator();
                while (keysetIt.hasNext()) {
                    keyset.push(keysetIt.next().value);
                }
            }
            keysets.push(keyset);
        }
        return keysets;
    }
    // [party index][key index]
    getSignatures() {
        const signatureSets = [];
        const witnesses = this.getId(schema_1.XyoObjectSchema.WITNESS.id);
        for (const fetter of witnesses) {
            const signatureSet = [];
            const partySignatureSets = fetter.getId(schema_1.XyoObjectSchema.SIGNATURE_SET.id);
            for (const partySignatureSet of partySignatureSets) {
                const signatureSetIt = partySignatureSet.newIterator();
                while (signatureSetIt.hasNext()) {
                    signatureSet.push(signatureSetIt.next().value);
                }
            }
            signatureSets.push(signatureSet);
        }
        return signatureSets;
    }
    // [party index][key index]
    getHeuristics() {
        const heuristics = [];
        const fetters = this.getId(schema_1.XyoObjectSchema.FETTER.id);
        for (const fetter of fetters) {
            const partyHeuristics = [];
            const fetterIt = fetter.newIterator();
            while (fetterIt.hasNext()) {
                partyHeuristics.push(fetterIt.next().value);
            }
            heuristics.push(partyHeuristics);
        }
        return heuristics;
    }
    getNumberOfFetters() {
        return this.getId(schema_1.XyoObjectSchema.FETTER.id).length;
    }
    getNumberOfWitnesses() {
        return this.getId(schema_1.XyoObjectSchema.WITNESS.id).length;
    }
    getHash(hasher) {
        return hasher.hash(this.getSigningData());
    }
    sign(signer) {
        return signer.sign(this.getSigningData());
    }
    getSigningData() {
        const bounds = this.getWitnessFetterBoundary();
        return this.getValue().getContentsCopy().slice(0, bounds);
    }
    getNumberOfParties() {
        const numberOfFetters = this.getNumberOfFetters();
        const numberOfWitnesses = this.getNumberOfWitnesses();
        if (numberOfFetters === numberOfWitnesses) {
            return numberOfWitnesses;
        }
        return undefined;
    }
    getFetterOfParty(partyIndex) {
        const numberOfParties = this.getNumberOfParties();
        if (numberOfParties) {
            if (numberOfParties <= partyIndex) {
                return undefined;
            }
            return this.get(partyIndex);
        }
        return undefined;
    }
    getWitnessOfParty(partyIndex) {
        const numberOfParties = this.getNumberOfParties();
        if (numberOfParties) {
            if (numberOfParties <= partyIndex) {
                return undefined;
            }
            return this.get(numberOfParties * 2 - (partyIndex + 1));
        }
        return undefined;
    }
    addToLedger(item) {
        if (this.getIsCompleted()) {
            throw new Error('Bound witness is completed');
        }
        this.addElement(item);
    }
    getWitnessFetterBoundary() {
        const fetters = this.getId(schema_1.XyoObjectSchema.FETTER.id);
        let offsetIndex = 0;
        for (const fetter of fetters) {
            offsetIndex += fetter.getAll().getSize();
        }
        return offsetIndex;
    }
}
exports.XyoBoundWitness = XyoBoundWitness;
//# sourceMappingURL=xyo-bound-witness.js.map