"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoZigZagBoundWitnessHander = void 0;
const xyo_catalog_flags_1 = require("../network/xyo-catalog-flags");
const xyo_choice_packet_1 = require("../network/xyo-choice-packet");
const object_model_1 = require("../object-model");
const xyo_bound_witness_handler_1 = __importDefault(require("./xyo-bound-witness-handler"));
const xyo_zig_zag_bound_witness_session_1 = require("./xyo-zig-zag-bound-witness-session");
class XyoZigZagBoundWitnessHander extends xyo_bound_witness_handler_1.default {
    constructor(payloadProvider) {
        super();
        this.payloadProvider = payloadProvider;
    }
    boundWitness(handler, catalogue, signers) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.currentBoundWitnessSession !== undefined) {
                    throw new Error('Bound witness is already in session');
                }
                const initData = handler.pipe.getInitiationData();
                if (initData) {
                    const serverChoice = catalogue.choose(initData.getChoice());
                    const cBw = yield this.handleBoundWitness(undefined, handler, xyo_catalog_flags_1.XyoCatalogFlags.flip(serverChoice), signers);
                    return cBw;
                }
                const response = yield handler.sendCatalogPacket(catalogue.getEncodedCanDo());
                if (!response) {
                    throw new Error('Response is undefined');
                }
                const adv = new xyo_choice_packet_1.XyoChoicePacket(response);
                const startingData = new object_model_1.XyoIterableStructure(new object_model_1.XyoBuffer(adv.getResponse()));
                const choice = adv.getChoice();
                const bw = yield this.handleBoundWitness(startingData, handler, choice, signers);
                return bw;
            }
            catch (error) {
                this.currentBoundWitnessSession = undefined;
                return undefined;
            }
        });
    }
    handleBoundWitness(startingData, handler, choice, signers) {
        return __awaiter(this, void 0, void 0, function* () {
            const payloads = yield this.payloadProvider.getPayloads(choice);
            const boundWitness = new xyo_zig_zag_bound_witness_session_1.XyoZigZagBoundWitnessSession(handler, payloads.signed, payloads.unsigned, signers, xyo_catalog_flags_1.XyoCatalogFlags.flip(choice));
            this.currentBoundWitnessSession = boundWitness;
            yield boundWitness.doBoundWitness(startingData);
            this.currentBoundWitnessSession = undefined;
            return boundWitness;
        });
    }
}
exports.XyoZigZagBoundWitnessHander = XyoZigZagBoundWitnessHander;
//# sourceMappingURL=xyo-zig-zag-bound-witness-handler.js.map