"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoZigZagBoundWitness = void 0;
const object_model_1 = require("../object-model");
const schema_1 = require("../schema");
const xyo_bound_witness_1 = require("./xyo-bound-witness");
class XyoZigZagBoundWitness extends xyo_bound_witness_1.XyoBoundWitness {
    constructor(signers, signedPayload, unsignedPayload) {
        super(object_model_1.XyoIterableStructure.newIterable(schema_1.XyoObjectSchema.BW, []).getAll());
        this.hasSentFetter = false;
        this.signers = signers;
        this.signedPayload = signedPayload;
        this.unsignedPayload = unsignedPayload;
    }
    incomingData(transfer, endpoint) {
        if (transfer) {
            this.addTransfer(transfer);
        }
        if (!this.hasSentFetter) {
            const fetter = xyo_bound_witness_1.XyoBoundWitness.createMasterArrayWithSubArray(schema_1.XyoObjectSchema.FETTER, schema_1.XyoObjectSchema.KEY_SET, this.signedPayload, this.getPublicKeysOfSigners());
            this.addToLedger(fetter);
            this.hasSentFetter = true;
        }
        if (this.getNumberOfFetters() !== this.getNumberOfWitnesses()) {
            return this.getReturnFromIncoming(this.getNumberOfWitnessesFromTransfer(transfer), endpoint);
        }
        return this.encodeTransfer([]);
    }
    getNumberOfWitnessesFromTransfer(transfer) {
        if (transfer) {
            return transfer.getId(schema_1.XyoObjectSchema.WITNESS.id).length;
        }
        return 0;
    }
    getReturnFromIncoming(numberOfWitnesses, endpoint) {
        if (numberOfWitnesses === 0 && !endpoint) {
            const elements = [];
            const it = this.newIterator();
            while (it.hasNext()) {
                elements.push(it.next().value);
            }
            return this.encodeTransfer(elements);
        }
        return this.passAndSign(numberOfWitnesses);
    }
    passAndSign(numberOfWitnesses) {
        const toSendBack = [];
        this.signBoundWitness(this.unsignedPayload);
        const fetters = this.getId(schema_1.XyoObjectSchema.FETTER.id);
        const witnesses = this.getId(schema_1.XyoObjectSchema.WITNESS.id);
        const x = numberOfWitnesses + 1;
        const y = fetters.length - 1;
        if (x <= y) {
            for (let i = x; i <= y; i++) {
                toSendBack.push(fetters[i]);
            }
        }
        toSendBack.push(witnesses[witnesses.length - 1]);
        return this.encodeTransfer(toSendBack);
    }
    encodeTransfer(items) {
        const fetters = [];
        const witness = [];
        for (const item of items) {
            switch (item.getSchema().id) {
                case schema_1.XyoObjectSchema.FETTER.id:
                    fetters.push(item);
                    break;
                case schema_1.XyoObjectSchema.WITNESS.id:
                    witness.push(item);
                    break;
                default:
                    throw new Error('Must be fetter or witness');
            }
        }
        return this.encodeFettersAndWitnessesForTransfer(fetters, witness, items);
    }
    encodeFettersAndWitnessesForTransfer(fetters, witness, items) {
        if (fetters.length === 0 && witness.length !== 0) {
            return object_model_1.XyoIterableStructure.newIterable(schema_1.XyoObjectSchema.WITNESS_SET, witness);
        }
        if (fetters.length !== 0 && witness.length === 0) {
            return object_model_1.XyoIterableStructure.newIterable(schema_1.XyoObjectSchema.FETTER_SET, fetters);
        }
        return object_model_1.XyoIterableStructure.newIterable(schema_1.XyoObjectSchema.BW_FRAGMENT, items);
    }
    addTransfer(transfer) {
        object_model_1.XyoIterableStructure.validate(transfer);
        const it = transfer.newIterator();
        while (it.hasNext()) {
            this.addToLedger(it.next().value);
        }
    }
    getPublicKeysOfSigners() {
        const publicKeys = [];
        for (const signer of this.signers) {
            publicKeys.push(signer.getPublicKey());
        }
        return publicKeys;
    }
    signBoundWitness(_payload) {
        const signatures = [];
        for (const signer of this.signers) {
            signatures.push(this.sign(signer));
        }
        const witness = xyo_bound_witness_1.XyoBoundWitness.createMasterArrayWithSubArray(schema_1.XyoObjectSchema.WITNESS, schema_1.XyoObjectSchema.SIGNATURE_SET, this.unsignedPayload, signatures);
        this.addToLedger(witness);
    }
}
exports.XyoZigZagBoundWitness = XyoZigZagBoundWitness;
//# sourceMappingURL=xyo-zig-zag-bound-witness.js.map