"use strict";
/*
 * File: index copy.ts
 * Project: @xyo-network/sdk-core-nodejs
 * File Created: Friday, 13th November 2020 2:27:23 pm
 * Author: XYO Development Team (support@xyo.network)
 * -----
 * Last Modified: Friday, 13th November 2020 2:31:15 pm
 * Modified By: XYO Development Team (support@xyo.network>)
 * -----
 * Copyright 2017 - 2020 XY - The Persistent Company
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bridgeHashSetResolver = exports.boundWitnessResolver = exports.previousHashResolver = exports.signatureSetResolver = exports.keySetResolver = exports.witnessResolver = exports.fetterResolver = exports.gpsResolver = exports.timeResolver = exports.lngResolver = exports.latResolver = exports.sha3Resolver = exports.sha256Resolver = exports.stubHashResolver = exports.stubPublicKey = exports.rsaPublicKeyResolver = exports.stubSignatureResolver = exports.rssiAt1MResolver = exports.rssiResolver = exports.indexResolver = exports.secp256K1PublicKeyResolver = exports.rsaSignatureResolver = exports.secp256K1SignatureResolver = exports.paymentKeyResolver = exports.nextPublicKeyResolver = void 0;
const bs58_1 = __importDefault(require("bs58"));
const object_model_1 = require("../../object-model");
const xyo_heuristic_resolver_1 = require("../xyo-heuristic-resolver");
const xyo_heuristics_common_util_1 = require("./xyo-heuristics-common-util");
exports.nextPublicKeyResolver = {
    resolve(heuristic) {
        const key = new object_model_1.XyoIterableStructure(heuristic).get(0);
        return {
            name: 'nextPublicKey',
            value: xyo_heuristic_resolver_1.XyoHumanHeuristicResolver.resolve(key.getAll().getContentsCopy()),
        };
    },
};
exports.paymentKeyResolver = {
    resolve(heuristic) {
        //const key = new XyoIterableStructure(heuristic).get(0)
        return {
            name: 'paymentKey',
            value: heuristic.toString('base64'),
        };
    },
};
exports.secp256K1SignatureResolver = {
    resolve(heuristic) {
        return {
            name: 'secp566k1Signature',
            value: heuristic.toString('base64'),
        };
    },
};
exports.rsaSignatureResolver = {
    resolve(heuristic) {
        return {
            name: 'rsaSignature',
            value: heuristic.toString('base64'),
        };
    },
};
exports.secp256K1PublicKeyResolver = {
    resolve(heuristic) {
        return {
            name: 'secp566k1PublicKey',
            value: bs58_1.default.encode(heuristic),
        };
    },
};
exports.indexResolver = {
    resolve(heuristic) {
        return {
            name: 'index',
            value: xyo_heuristics_common_util_1.readUnsignedNumber(new object_model_1.XyoStructure(heuristic).getValue().getContentsCopy()),
        };
    },
};
exports.rssiResolver = {
    resolve(heuristic) {
        return {
            name: 'rssi',
            value: xyo_heuristics_common_util_1.readSignedNumber(new object_model_1.XyoStructure(heuristic).getValue().getContentsCopy()),
        };
    },
};
exports.rssiAt1MResolver = {
    resolve(heuristic) {
        return {
            name: 'rssiAt1m',
            value: xyo_heuristics_common_util_1.readSignedNumber(new object_model_1.XyoStructure(heuristic).getValue().getContentsCopy()),
        };
    },
};
exports.stubSignatureResolver = {
    resolve(heuristic) {
        return {
            name: 'stubSignature',
            value: heuristic.toString('base64'),
        };
    },
};
exports.rsaPublicKeyResolver = {
    resolve(heuristic) {
        return {
            name: 'rsaPublicKey',
            value: bs58_1.default.encode(heuristic),
        };
    },
};
exports.stubPublicKey = {
    resolve(heuristic) {
        return {
            name: 'stubPublicKey',
            value: bs58_1.default.encode(heuristic),
        };
    },
};
exports.stubHashResolver = {
    resolve(heuristic) {
        return {
            name: 'stubHash',
            value: bs58_1.default.encode(heuristic),
        };
    },
};
exports.sha256Resolver = {
    resolve(heuristic) {
        return {
            name: 'sha256',
            value: bs58_1.default.encode(heuristic),
        };
    },
};
exports.sha3Resolver = {
    resolve(heuristic) {
        return {
            name: 'sha3',
            value: bs58_1.default.encode(heuristic),
        };
    },
};
exports.latResolver = {
    resolve(heuristic) {
        const value = new object_model_1.XyoStructure(heuristic).getValue().getContentsCopy();
        return {
            name: 'lat',
            value: value.readDoubleBE(0),
        };
    },
};
exports.lngResolver = {
    resolve(heuristic) {
        const value = new object_model_1.XyoStructure(heuristic).getValue().getContentsCopy();
        return {
            name: 'lng',
            value: value.readDoubleBE(0),
        };
    },
};
exports.timeResolver = {
    resolve(heuristic) {
        const value = new object_model_1.XyoStructure(heuristic).getValue().getContentsCopy();
        return {
            name: 'date',
            value: value.readUIntBE(2, 6),
        };
    },
};
exports.gpsResolver = {
    resolve(heuristic) {
        return {
            name: 'gps',
            value: xyo_heuristics_common_util_1.uniqueIterableToObject(new object_model_1.XyoIterableStructure(heuristic)),
        };
    },
};
exports.fetterResolver = {
    resolve(heuristic) {
        return {
            name: 'fetter',
            value: xyo_heuristics_common_util_1.uniqueIterableToObject(new object_model_1.XyoIterableStructure(heuristic)),
        };
    },
};
exports.witnessResolver = {
    resolve(heuristic) {
        return {
            name: 'witness',
            value: xyo_heuristics_common_util_1.uniqueIterableToObject(new object_model_1.XyoIterableStructure(heuristic)),
        };
    },
};
exports.keySetResolver = {
    resolve(heuristic) {
        return {
            name: 'keySet',
            value: xyo_heuristics_common_util_1.iterableObjectToArray(new object_model_1.XyoIterableStructure(heuristic)),
        };
    },
};
exports.signatureSetResolver = {
    resolve(heuristic) {
        return {
            name: 'signatureSet',
            value: xyo_heuristics_common_util_1.iterableObjectToArray(new object_model_1.XyoIterableStructure(heuristic)),
        };
    },
};
exports.previousHashResolver = {
    resolve(heuristic) {
        const children = xyo_heuristics_common_util_1.iterableObjectToArray(new object_model_1.XyoIterableStructure(heuristic));
        if (children.length !== 1) {
            // we expect the previous hash to be a length of 1
            return {
                name: 'previousHash',
                value: 'invalid',
            };
        }
        return {
            name: 'previousHash',
            value: children[0].value,
        };
    },
};
exports.boundWitnessResolver = {
    resolve(heuristic) {
        return {
            name: 'boundWitness',
            value: xyo_heuristics_common_util_1.iterableObjectToArray(new object_model_1.XyoIterableStructure(heuristic)).map((h) => {
                return h.value;
            }),
        };
    },
};
exports.bridgeHashSetResolver = {
    resolve(heuristic) {
        return {
            name: 'bridgeHashSet',
            value: xyo_heuristics_common_util_1.iterableObjectToArray(new object_model_1.XyoIterableStructure(heuristic)).map((h) => {
                return h.value;
            }),
        };
    },
};
//# sourceMappingURL=resolvers.js.map