"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.iterableObjectToArray = exports.uniqueIterableToObject = exports.readUnsignedNumber = exports.readSignedNumber = void 0;
const xyo_heuristic_resolver_1 = require("../xyo-heuristic-resolver");
const readSignedNumber = (buffer) => {
    switch (buffer.length) {
        case 1:
            return buffer.readInt8(0);
        case 2:
            return buffer.readInt16BE(0);
        case 4:
            return buffer.readInt32BE(0);
    }
    return -1;
};
exports.readSignedNumber = readSignedNumber;
const readUnsignedNumber = (buffer) => {
    switch (buffer.length) {
        case 1:
            return buffer.readUInt8(0);
        case 2:
            return buffer.readUInt16BE(0);
        case 4:
            return buffer.readUInt32BE(0);
    }
    return -1;
};
exports.readUnsignedNumber = readUnsignedNumber;
const uniqueIterableToObject = (buffer) => {
    const it = buffer.newIterator();
    const map = {};
    while (it.hasNext()) {
        const child = it.next().value;
        const readable = xyo_heuristic_resolver_1.XyoHumanHeuristicResolver.resolve(child.getAll().getContentsCopy());
        map[readable.name] = readable.value;
    }
    return map;
};
exports.uniqueIterableToObject = uniqueIterableToObject;
const iterableObjectToArray = (buffer) => {
    const it = buffer.newIterator();
    const arr = [];
    while (it.hasNext()) {
        const child = it.next().value;
        const readable = xyo_heuristic_resolver_1.XyoHumanHeuristicResolver.resolve(child.getAll().getContentsCopy());
        arr.push(readable);
    }
    return arr;
};
exports.iterableObjectToArray = iterableObjectToArray;
//# sourceMappingURL=xyo-heuristics-common-util.js.map