"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoBoundWitnessOriginGetter = void 0;
const schema_1 = require("../schema");
const common_1 = require("./common");
class XyoBoundWitnessOriginGetter {
    static getOriginInformation(boundWitness) {
        const origins = [];
        const allHeuristics = boundWitness.getHeuristics();
        for (const fetterHeuristics of allHeuristics) {
            origins.push(XyoBoundWitnessOriginGetter.getOriginInformationFromFetter(fetterHeuristics));
        }
        return origins;
    }
    static getOriginInformationFromFetter(fetterHeuristics) {
        let nextPublicKey;
        let previousHash;
        let index = -1;
        for (const heuristic of fetterHeuristics) {
            if (heuristic.getSchema().id === schema_1.XyoObjectSchema.NEXT_PUBLIC_KEY.id) {
                const nextPublicKeyArray = heuristic;
                if (nextPublicKeyArray.getCount() !== 1) {
                    throw new Error('1 next public key expected');
                }
                nextPublicKey = nextPublicKeyArray.get(0).getAll().getContentsCopy();
            }
            if (heuristic.getSchema().id === schema_1.XyoObjectSchema.PREVIOUS_HASH.id) {
                const previousHashArray = heuristic;
                if (previousHashArray.getCount() !== 1) {
                    throw new Error('1 hash expected in previous hash');
                }
                previousHash = previousHashArray.get(0).getAll().getContentsCopy();
            }
            if (heuristic.getSchema().id === schema_1.XyoObjectSchema.INDEX.id) {
                index = common_1.indexResolver.resolve(heuristic.getAll().getContentsCopy())
                    .value;
            }
        }
        return {
            index,
            nextPublicKey,
            previousHash,
        };
    }
}
exports.XyoBoundWitnessOriginGetter = XyoBoundWitnessOriginGetter;
//# sourceMappingURL=xyo-bound-witness-origin.js.map