"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoTcpPipe = void 0;
const sdk_base_nodejs_1 = require("@xyo-network/sdk-base-nodejs");
class XyoTcpPipe extends sdk_base_nodejs_1.XyoBase {
    constructor(socket, initData) {
        super();
        this.socket = socket;
        this.initData = initData;
    }
    getInitiationData() {
        return this.initData;
    }
    send(data, waitForResponse) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logVerbose(`Sending data through socket: ${data.toString('hex')}`);
            yield this.sendData(data);
            if (waitForResponse) {
                const response = yield this.waitForMessage();
                this.logVerbose(`Got data through socket: ${response.toString('hex')}`);
                return response;
            }
            return undefined;
        });
    }
    // eslint-disable-next-line require-await
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logInfo(`Closing connection with ${this.socket.remoteAddress}:${this.socket.remotePort}`);
            this.socket.end();
            setTimeout(() => {
                if (!this.socket.destroyed) {
                    this.socket.destroy();
                }
            }, 2000);
        });
    }
    waitForMessage() {
        return new Promise((resolve, reject) => {
            let hasResumed = false;
            let waitSize;
            let currentSize = 0;
            let currentBuffer = Buffer.alloc(0);
            const cleanup = () => {
                this.socket.removeAllListeners('data');
                this.socket.removeAllListeners('close');
                this.socket.removeAllListeners('end');
            };
            const onTimeout = () => {
                if (!hasResumed) {
                    hasResumed = true;
                    this.socket.destroy();
                    cleanup();
                    reject(new Error('timeout'));
                }
            };
            const onClose = () => {
                if (!hasResumed) {
                    hasResumed = true;
                    cleanup();
                    reject(new Error('Socket closed while waiting for write'));
                }
            };
            this.socket.on('data', (data) => {
                currentSize += data.length;
                currentBuffer = Buffer.concat([currentBuffer, data]);
                if (waitSize === undefined && currentSize >= 4) {
                    waitSize = currentBuffer.readUInt32BE(0);
                }
                if (currentSize >= waitSize && !hasResumed) {
                    hasResumed = true;
                    cleanup();
                    resolve(currentBuffer.slice(4));
                }
            });
            this.socket.on('close', onClose);
            this.socket.on('error', onClose);
            setTimeout(onTimeout, 7500);
        });
    }
    sendData(data) {
        return new Promise((resolve, reject) => {
            const sizeBuffer = Buffer.alloc(4);
            sizeBuffer.writeUInt32BE(data.length + 4, 0);
            const dataWithSize = Buffer.concat([sizeBuffer, data]);
            this.socket.write(dataWithSize, (error) => {
                if (error === undefined) {
                    resolve();
                }
                else {
                    reject(error);
                }
            });
        });
    }
}
exports.XyoTcpPipe = XyoTcpPipe;
//# sourceMappingURL=xyo-tcp-pipe.js.map