"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoServerTcpNetwork = void 0;
const sdk_base_nodejs_1 = require("@xyo-network/sdk-base-nodejs");
const net_1 = __importDefault(require("net"));
const xyo_advertise_packet_1 = require("../xyo-advertise-packet");
const xyo_tcp_pipe_1 = require("./xyo-tcp-pipe");
class XyoServerTcpNetwork extends sdk_base_nodejs_1.XyoBase {
    constructor(port) {
        super();
        this.port = port;
        this.server = net_1.default.createServer(this.connectionListener.bind(this));
        this.server.on('error', (e) => {
            this.logWarning(`Unknown server socket error: ${e}`);
            if (!this.server.listening) {
                this.logInfo(`Starting listing again on port ${this.port}`);
                this.server.listen(this.port);
            }
        });
    }
    startListening() {
        this.server.listen(this.port);
    }
    stopListening() {
        this.server.close();
    }
    connectionListener(socket) {
        this.logInfo(`New connection made with ${socket.remoteAddress}:${socket.remotePort}`);
        socket.on('error', (e) => {
            this.logWarning(`Unknown socket error: ${e}`);
            socket.destroy();
        });
        let waitSize;
        let currentSize = 0;
        let currentBuffer = Buffer.alloc(0);
        const cleanup = () => {
            socket.removeAllListeners('data');
            socket.removeAllListeners('close');
            socket.removeAllListeners('end');
            socket.removeAllListeners('timeout');
            socket.destroy();
        };
        const timeout = setTimeout(cleanup, 5000);
        socket.on('timeout', () => {
            cleanup();
        });
        socket.on('data', (data) => {
            currentSize += data.length;
            currentBuffer = Buffer.concat([currentBuffer, data]);
            if (waitSize === undefined && currentSize >= 4) {
                waitSize = currentBuffer.readUInt32BE(0);
            }
            if (currentSize >= waitSize) {
                clearTimeout(timeout);
                if (!this.onInternalPipeCreated(socket, currentBuffer.slice(4))) {
                    cleanup();
                }
                socket.removeAllListeners('data');
                socket.removeAllListeners('close');
                socket.removeAllListeners('end');
            }
        });
        socket.on('close', cleanup);
        socket.on('end', cleanup);
        socket.setTimeout(1000 * 60 * 1); // 1 min
    }
    onInternalPipeCreated(socket, data) {
        const socketPipe = new xyo_tcp_pipe_1.XyoTcpPipe(socket, new xyo_advertise_packet_1.XyoAdvertisePacket(data));
        const callback = this.onPipeCreated;
        if (callback) {
            return callback(socketPipe);
        }
        return false;
    }
}
exports.XyoServerTcpNetwork = XyoServerTcpNetwork;
//# sourceMappingURL=xyo-tcp-server-network.js.map