"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoBuffer = void 0;
class XyoBuffer {
    constructor(buffer, startOffset, endOffset) {
        this.rootBuffer = buffer;
        this.startOffset = startOffset || 0;
        this.endOffset = endOffset || buffer.length;
    }
    static wrap(buffer, startOffset, endOffset) {
        return new XyoBuffer(buffer.rootBuffer, startOffset + buffer.startOffset, endOffset + buffer.startOffset);
    }
    getSize() {
        return this.endOffset - this.startOffset;
    }
    getUInt8(offset) {
        return this.rootBuffer.readUInt8(this.startOffset + offset);
    }
    getUInt16BE(offset) {
        return this.rootBuffer.readUInt16BE(this.startOffset + offset);
    }
    getUInt32BE(offset) {
        return this.rootBuffer.readUInt32BE(this.startOffset + offset);
    }
    getUInt64BE(offset) {
        // todo find way to get the int64 size, this will get all the way up to a 2^32 as a size
        return this.rootBuffer.readUInt32BE(this.startOffset + offset + 4);
    }
    getContentsCopy() {
        return this.rootBuffer.slice(this.startOffset, this.endOffset);
    }
    copyRangeOf(from, to) {
        return XyoBuffer.wrap(this, from, to);
    }
}
exports.XyoBuffer = XyoBuffer;
//# sourceMappingURL=xyo-buffer.js.map