"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoSchema = void 0;
const xyo_size_1 = require("./xyo-size");
class XyoSchema {
    constructor(id, encodingCatalogue) {
        this.id = id;
        this.encodingCatalogue = encodingCatalogue;
    }
    static create(id, isIterable, isTyped, sizeIdentifier) {
        const iterableByte = XyoSchema.getIterableByte(isIterable);
        const getTypedByte = XyoSchema.getTypedByte(isTyped);
        const getSizeByte = XyoSchema.getSizeByte(sizeIdentifier);
        const encodingCatalogue = iterableByte | getTypedByte | getSizeByte;
        return new XyoSchema(id, encodingCatalogue);
    }
    static getIterableByte(isIterable) {
        if (isIterable) {
            return 0x20;
        }
        return 0x00;
    }
    static getTypedByte(isTyped) {
        if (isTyped) {
            return 0x10;
        }
        return 0x00;
    }
    static getSizeByte(size) {
        switch (size) {
            case xyo_size_1.XyoSize.ONE:
                return 0x00;
            case xyo_size_1.XyoSize.TWO:
                return 0x40;
            case xyo_size_1.XyoSize.FOUR:
                return 0x80;
            case xyo_size_1.XyoSize.EIGHT:
                return 0xc0;
        }
    }
    getSizeIdentifier() {
        if ((this.encodingCatalogue & 0xc0) === 0x00) {
            return xyo_size_1.XyoSize.ONE;
        }
        if ((this.encodingCatalogue & 0xc0) === 0x40) {
            return xyo_size_1.XyoSize.TWO;
        }
        if ((this.encodingCatalogue & 0xc0) === 0x80) {
            return xyo_size_1.XyoSize.FOUR;
        }
        return xyo_size_1.XyoSize.EIGHT;
    }
    getIsIterable() {
        return (this.encodingCatalogue & 0x20) !== 0;
    }
    getIsTypedIterable() {
        return (this.encodingCatalogue & 0x10) !== 0;
    }
}
exports.XyoSchema = XyoSchema;
//# sourceMappingURL=xyo-schema.js.map