"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoBoundWitnessInserter = void 0;
const sdk_base_nodejs_1 = require("@xyo-network/sdk-base-nodejs");
const bs58_1 = __importDefault(require("bs58"));
const object_model_1 = require("../object-model");
const schema_1 = require("../schema");
class XyoBoundWitnessInserter extends sdk_base_nodejs_1.XyoBase {
    constructor(hasher, state, blockRepo) {
        super();
        this.blockListeners = {};
        this.hasher = hasher;
        this.state = state;
        this.blockRepository = blockRepo;
    }
    addBlockListener(key, listener) {
        this.blockListeners[key] = listener;
    }
    removeBlockListener(key) {
        delete this.blockListeners[key];
    }
    insert(boundWitness) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridgeBlocks = this.getNestedObjectType(boundWitness, schema_1.XyoObjectSchema.WITNESS, schema_1.XyoObjectSchema.BRIDGE_BLOCK_SET);
            const bridgeBlocksHashes = this.getNestedObjectType(boundWitness, schema_1.XyoObjectSchema.FETTER, schema_1.XyoObjectSchema.BRIDGE_HASH_SET);
            const rootBlockWithoutBridgedBlocks = this.removeBridgeBlocks(boundWitness);
            const hash = boundWitness.getHash(this.hasher);
            this.state.addOriginBlock(hash);
            this.logInfo(`Inserted new origin block with hash: ${bs58_1.default.encode(hash.getAll().getContentsCopy())}`);
            yield this.state.repo.commit();
            // no need to await the add block, this can be async
            this.blockRepository.addOriginBlock(hash.getAll().getContentsCopy(), rootBlockWithoutBridgedBlocks.getAll().getContentsCopy());
            this.logInfo(`Origin state at new height: ${this.state.getIndexAsNumber()}`);
            if (bridgeBlocks && bridgeBlocksHashes) {
                // no need to await the add block, this can be async
                this.blockRepository.addOriginBlocks(bridgeBlocksHashes.getAll().getContentsCopy(), bridgeBlocks.getAll().getContentsCopy());
            }
            for (const [, value] of Object.entries(this.blockListeners)) {
                value(boundWitness.getAll().getContentsCopy());
            }
        });
    }
    getNestedObjectType(boundWitness, rootSchema, subSchema) {
        const it = boundWitness.newIterator();
        while (it.hasNext()) {
            const bwItem = it.next().value;
            if (bwItem.getSchema().id === rootSchema.id &&
                bwItem instanceof object_model_1.XyoIterableStructure) {
                const fetterIt = bwItem.newIterator();
                while (fetterIt.hasNext()) {
                    const fetterItem = fetterIt.next().value;
                    if (fetterItem.getSchema().id === subSchema.id) {
                        return fetterItem;
                    }
                }
            }
        }
        return;
    }
    removeBridgeBlocks(boundWitness) {
        const newLedgerItems = [];
        const it = boundWitness.newIterator();
        while (it.hasNext()) {
            const bwItem = it.next().value;
            if (bwItem.getSchema().id === schema_1.XyoObjectSchema.WITNESS.id &&
                bwItem instanceof object_model_1.XyoIterableStructure) {
                const witnessIt = bwItem.newIterator();
                const newWitnessItems = [];
                while (witnessIt.hasNext()) {
                    const witnessItem = witnessIt.next().value;
                    if (witnessItem.getSchema().id !== schema_1.XyoObjectSchema.BRIDGE_BLOCK_SET.id) {
                        newWitnessItems.push(witnessItem);
                    }
                }
                newLedgerItems.push(object_model_1.XyoIterableStructure.newIterable(schema_1.XyoObjectSchema.WITNESS, newWitnessItems));
            }
            else {
                newLedgerItems.push(bwItem);
            }
        }
        return object_model_1.XyoIterableStructure.newIterable(schema_1.XyoObjectSchema.BW, newLedgerItems);
    }
}
exports.XyoBoundWitnessInserter = XyoBoundWitnessInserter;
//# sourceMappingURL=xyo-bound-witness-inserter.js.map