"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoOriginState = void 0;
const object_model_1 = require("../object-model");
const schema_1 = require("../schema");
class XyoOriginState {
    constructor(repo) {
        this.waitingSigners = [];
        this.repo = repo;
    }
    static createNextPublicKey(publicKey) {
        return object_model_1.XyoStructure.newInstance(schema_1.XyoObjectSchema.NEXT_PUBLIC_KEY, publicKey.getAll());
    }
    static createPreviousHash(hash) {
        return object_model_1.XyoIterableStructure.newIterable(schema_1.XyoObjectSchema.PREVIOUS_HASH, [
            hash,
        ]);
    }
    static createIndex(index) {
        const numberBuffer = Buffer.alloc(4);
        numberBuffer.writeUInt32BE(index, 0);
        return object_model_1.XyoStructure.newInstance(schema_1.XyoObjectSchema.INDEX, new object_model_1.XyoBuffer(numberBuffer));
    }
    getIndexAsNumber() {
        return this.getIndex().getValue().getContentsCopy().readUInt32BE(0);
    }
    getNextPublicKey() {
        return this.nextPublicKey;
    }
    getIndex() {
        const indexInStore = this.repo.getIndex();
        if (indexInStore) {
            return indexInStore;
        }
        return XyoOriginState.createIndex(0);
    }
    getPreviousHash() {
        return this.repo.getPreviousHash();
    }
    removeOldestSigner() {
        this.repo.removeOldestSigner();
    }
    getSigners() {
        return this.repo.getSigners();
    }
    addSigner(signer) {
        const index = this.getIndex().getValue().getContentsCopy().readUInt32BE(0);
        if (index === 0) {
            this.repo.addSigner(signer);
            return;
        }
        this.waitingSigners.push(signer);
        this.nextPublicKey = XyoOriginState.createNextPublicKey(signer.getPublicKey());
    }
    addOriginBlock(hash) {
        const previousHash = XyoOriginState.createPreviousHash(hash);
        this.nextPublicKey = undefined;
        this.addWaitingSigner();
        this.repo.putPreviousHash(previousHash);
        this.incrementIndex();
    }
    addWaitingSigner() {
        if (this.waitingSigners.length > 0) {
            this.repo.addSigner(this.waitingSigners[0]);
            this.waitingSigners.shift();
        }
    }
    incrementIndex() {
        const index = this.getIndex().getValue().getContentsCopy().readUInt32BE(0);
        const newIndex = XyoOriginState.createIndex(index + 1);
        this.repo.putIndex(newIndex);
    }
}
exports.XyoOriginState = XyoOriginState;
//# sourceMappingURL=xyo-origin-state.js.map