"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoFileOriginStateRepository = void 0;
const fs_1 = __importDefault(require("fs"));
const object_model_1 = require("../object-model");
class XyoFileOriginStateRepository {
    constructor(statePath) {
        this.signersCache = [];
        this.statePath = statePath;
    }
    addSigner(signer) {
        this.signersCache.push(signer);
    }
    removeOldestSigner() {
        if (this.signersCache.length > 0) {
            this.signersCache.shift();
        }
    }
    putIndex(index) {
        this.indexCache = index;
    }
    putPreviousHash(previousHash) {
        this.previousHashCache = previousHash;
    }
    getIndex() {
        return this.indexCache;
    }
    getPreviousHash() {
        return this.previousHashCache;
    }
    getSigners() {
        if (this.signersCache) {
            return this.signersCache;
        }
        return [];
    }
    commit() {
        const state = this.getCurrentFileState();
        return new Promise((resolve, reject) => {
            fs_1.default.writeFile(this.statePath, JSON.stringify(state), 'utf8', (error) => {
                if (error) {
                    reject(error);
                }
                else {
                    resolve();
                }
            });
        });
    }
    restore(signerFromPrivateKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentState = yield this.readCurrentFileState();
            if (currentState) {
                if (currentState.index) {
                    this.indexCache = new object_model_1.XyoStructure(new object_model_1.XyoBuffer(Buffer.from(currentState.index, 'base64')));
                }
                if (currentState.previousHash) {
                    this.previousHashCache = new object_model_1.XyoStructure(new object_model_1.XyoBuffer(Buffer.from(currentState.previousHash, 'base64')));
                }
                currentState.signers.forEach((privateKey) => {
                    this.signersCache.push(signerFromPrivateKey(Buffer.from(privateKey, 'base64')));
                });
            }
        });
    }
    readCurrentFileState() {
        return new Promise((resolve, _reject) => {
            fs_1.default.readFile(this.statePath, (error, data) => {
                if (error) {
                    resolve(undefined);
                    return;
                }
                const stringValue = data.toString('utf8');
                resolve(JSON.parse(stringValue));
            });
        });
    }
    getCurrentFileState() {
        const signersString = [];
        let previousHashString;
        let indexString;
        if (this.previousHashCache) {
            previousHashString = this.previousHashCache
                .getAll()
                .getContentsCopy()
                .toString('base64');
        }
        if (this.indexCache) {
            indexString = this.indexCache
                .getAll()
                .getContentsCopy()
                .toString('base64');
        }
        this.signersCache.forEach((signer) => {
            signersString.push(signer.getPrivateKey().getAll().getContentsCopy().toString('base64'));
        });
        const state = {
            index: indexString,
            previousHash: previousHashString,
            signers: signersString,
        };
        return state;
    }
}
exports.XyoFileOriginStateRepository = XyoFileOriginStateRepository;
//# sourceMappingURL=xyo-file-origin-state-repository.js.map