"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoMemoryBlockRepository = void 0;
const object_model_1 = require("../object-model");
class XyoMemoryBlockRepository {
    constructor() {
        this.blockMapping = new Map();
    }
    // eslint-disable-next-line require-await
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            return true;
        });
    }
    // eslint-disable-next-line require-await
    removeOriginBlock(hash) {
        return __awaiter(this, void 0, void 0, function* () {
            this.blockMapping.delete(hash.toString('base64'));
        });
    }
    // eslint-disable-next-line require-await
    addOriginBlock(hash, block) {
        return __awaiter(this, void 0, void 0, function* () {
            this.blockMapping.set(hash.toString('base64'), block);
        });
    }
    // eslint-disable-next-line require-await
    getOriginBlock(hash) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.blockMapping.get(hash.toString('base64'));
        });
    }
    // eslint-disable-next-line require-await
    addOriginBlocks(hashes, blocks) {
        return __awaiter(this, void 0, void 0, function* () {
            const blockStructure = new object_model_1.XyoIterableStructure(blocks);
            const hashesStructure = new object_model_1.XyoIterableStructure(hashes);
            const blockIt = blockStructure.newIterator();
            const hashIt = hashesStructure.newIterator();
            while (blockIt.hasNext()) {
                const block = blockIt.next().value;
                const hash = hashIt.next().value;
                this.addOriginBlock(hash.getAll().getContentsCopy(), block.getAll().getContentsCopy());
            }
        });
    }
    // eslint-disable-next-line require-await
    getOriginBlocks(_limit, _offset) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('getOriginBlocks not implemented in XyoMemoryBlockRepository');
        });
    }
}
exports.XyoMemoryBlockRepository = XyoMemoryBlockRepository;
//# sourceMappingURL=xyo-memory-block-repository.js.map