"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoSecp2556k1 = void 0;
const elliptic_1 = __importDefault(require("elliptic"));
const object_model_1 = require("../../object-model");
const schema_1 = require("../../schema");
const ec = new elliptic_1.default.ec('secp256k1');
class XyoSecp2556k1 {
    constructor(key) {
        if (key) {
            const structure = new object_model_1.XyoStructure(key);
            const privateKey = structure.getValue().getContentsCopy();
            this.key = ec.keyFromPrivate(privateKey);
            return;
        }
        this.key = ec.genKeyPair();
    }
    static buildDerSignature(xyBuffer) {
        const sizeOfR = xyBuffer.readUInt8(0);
        const rBuffer = xyBuffer.slice(1, sizeOfR + 1);
        const source = Buffer.concat([
            Buffer.from([0x02]),
            xyBuffer.slice(0, 1),
            rBuffer,
            Buffer.from([0x02]),
            xyBuffer.slice(sizeOfR + 1),
        ]);
        const sourceBufferSizeBuffer = Buffer.alloc(1);
        sourceBufferSizeBuffer.writeUInt8(source.length, 0);
        return Buffer.concat([
            Buffer.from([0x30]),
            sourceBufferSizeBuffer,
            source,
        ]).toString('hex');
    }
    sign(data) {
        const signature = this.key.sign(data);
        const rBuffer = signature.r.toBuffer();
        const sBuffer = signature.s.toBuffer();
        const value = Buffer.concat([
            Buffer.from([rBuffer.length]),
            rBuffer,
            Buffer.from([sBuffer.length]),
            sBuffer,
        ]);
        return object_model_1.XyoStructure.newInstance(schema_1.XyoObjectSchema.EC_SIGNATURE, new object_model_1.XyoBuffer(value));
    }
    getPublicKey() {
        const key = this.key.getPublic();
        const x = key.getX().toBuffer();
        const y = key.getY().toBuffer();
        const buffer = Buffer.concat([
            this.writePointTo32ByteBuffer(x),
            this.writePointTo32ByteBuffer(y),
        ]);
        return object_model_1.XyoStructure.newInstance(schema_1.XyoObjectSchema.EC_PUBLIC_KEY, new object_model_1.XyoBuffer(buffer));
    }
    getPrivateKey() {
        const privateKey = this.key.getPrivate('hex').toString();
        const buffer = new object_model_1.XyoBuffer(Buffer.from(privateKey, 'hex'));
        return object_model_1.XyoStructure.newInstance(schema_1.XyoObjectSchema.EC_PRIVATE_KEY, buffer);
    }
    writePointTo32ByteBuffer(point) {
        if (point.length === 32) {
            return point;
        }
        const dest = Buffer.alloc(32);
        const offset = dest.length - point.length;
        let index = 0;
        while (index < point.length) {
            dest[offset + index] = point[index];
            index += 1;
        }
        return dest;
    }
}
exports.XyoSecp2556k1 = XyoSecp2556k1;
XyoSecp2556k1.verify = (publicKey, signature, data
// eslint-disable-next-line require-await
) => __awaiter(void 0, void 0, void 0, function* () {
    const signatureStructure = new object_model_1.XyoStructure(new object_model_1.XyoBuffer(signature));
    const publicKeyStructure = new object_model_1.XyoStructure(new object_model_1.XyoBuffer(publicKey));
    const derSignature = XyoSecp2556k1.buildDerSignature(signatureStructure.getValue().getContentsCopy());
    const x = publicKeyStructure.getValue().copyRangeOf(0, 31);
    const y = publicKeyStructure.getValue().copyRangeOf(32, 64);
    const hexKey = ['04', x, y].join('');
    const key = ec.keyFromPublic(hexKey, 'hex');
    return key.verify(data, derSignature);
});
//# sourceMappingURL=xyo-secp256k1.js.map