"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoOriginPayloadConstructor = void 0;
class XyoOriginPayloadConstructor {
    constructor(originState) {
        this.heuristicGetters = new Map();
        this.originState = originState;
    }
    addHeuristicGetter(key, getter) {
        this.heuristicGetters.set(key, getter);
    }
    removeHeuristicGetter(key) {
        this.heuristicGetters.delete(key);
    }
    // eslint-disable-next-line require-await
    getPayloads(choice) {
        return __awaiter(this, void 0, void 0, function* () {
            const originItems = this.getOriginItems();
            const getterItems = this.getAllHeuristics(choice);
            const payload = {
                signed: originItems.concat(getterItems),
                unsigned: [],
            };
            return payload;
        });
    }
    getAllHeuristics(choice) {
        const toReturn = [];
        this.heuristicGetters.forEach((value, _) => {
            const item = value.getHeuristic(choice);
            if (item) {
                toReturn.push(item);
            }
        });
        return toReturn;
    }
    getOriginItems() {
        const toReturn = [];
        const index = this.originState.getIndex();
        const previousHash = this.originState.getPreviousHash();
        const nextPublicKey = this.originState.getNextPublicKey();
        toReturn.push(index);
        if (previousHash) {
            toReturn.push(previousHash);
        }
        if (nextPublicKey) {
            toReturn.push(nextPublicKey);
        }
        return toReturn;
    }
}
exports.XyoOriginPayloadConstructor = XyoOriginPayloadConstructor;
//# sourceMappingURL=xyo-origin-payload-constructor.js.map